defmodule DecentralisedBookIndex.Metadata.Publisher do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer,
    extensions: [AshJsonApi.Resource],
    authorizers: [Ash.Policy.Authorizer]

  require Ash.Query
  alias DecentralisedBookIndex.Metadata

  json_api do
    type "publisher"
  end

  postgres do
    table "publishers"
    repo DecentralisedBookIndex.Repo

    custom_indexes do
      index "name gin_trgm_ops", name: "publisher_name_gin_index", using: "GIN"
    end
  end

  resource do
    description "An publisher's metadata."
  end

  actions do
    defaults [:destroy]

    create :create do
      primary? true
      accept [:name, :dbi_server_id]
    end

    create :sync_create do
      accept [:id, :name, :inserted_at, :updated_at, :dbi_server_id]
    end

    read :read do
      description "Return a list of Publishers."
      primary? true
    end

    read :by_id do
      description "Return the Publisher by its id."
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end

    read :search do
      description "Return a list of Publishers, optionally filtering by name."

      argument :name, :ci_string do
        description "Return Publishers, which names includes the name."
        constraints allow_empty?: true
        default ""
      end

      filter expr(contains(name, ^arg(:name)))

      pagination offset?: true, default_limit: 10
    end

    update :update do
      primary? true
      accept [:name]
    end

    update :assign_dbi_server_id do
      accept [:dbi_server_id]
    end

    update :sync do
      accept [:name, :inserted_at, :updated_at, :dbi_server_id]
    end
  end

  policies do
    bypass actor_attribute_equals(:role, :admin) do
      authorize_if always()
    end

    policy action_type(:read) do
      authorize_if always()
    end

    policy action_type(:create) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:update) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:destroy) do
      authorize_if actor_attribute_equals(:role, :admin)
    end
  end

  attributes do
    uuid_primary_key :id, writable?: true

    attribute :name, :string do
      allow_nil? false
      public? true
    end

    timestamps do
      writable? true
      public? true
    end
  end

  relationships do
    belongs_to :dbi_server, Metadata.DBIServer do
      public? true
    end

    has_many :books, Metadata.Book
  end
end
