defmodule DecentralisedBookIndexWeb.BookLive.Show do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata
  alias DecentralisedBookIndex.Accounts.Role

  @impl true
  def render(assigns) do
    ~H"""
    <.header class="flex-col lg:flex-row">
      {@book.title}
      <:subtitle>
        <.authors book={@book} />
      </:subtitle>

      <:actions>
        <%= if @current_user != nil and Role.can_moderate?(@current_user.role) do %>
          <div class="flex flex-row gap-2">
            <.add_button_link patch={~p"/books/#{@book}/new"}>
              Edition
            </.add_button_link>
            <%= if is_nil(@book.dbi_server) do %>
              <.link patch={~p"/books/#{@book}/edit"}>
                <.edit_button>
                  Edit
                </.edit_button>
              </.link>
            <% end %>
          </div>
        <% end %>
      </:actions>
    </.header>

    <div class="mb-4">
      <%= if @book.cover_image_url != nil do %>
        <img class="w-36 shadow-lg mt-14 mx-auto object-cover" src={@book.cover_image_url} alt={"#{@book.title} image"}/>
      <% else %>
        <div class="relative w-36 h-36 overflow-hidden mt-14 mx-auto">
          <svg class="w-36 h-36 text-gray-800 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor" viewBox="0 0 24 24">
              <path fill-rule="evenodd" d="M6 2a2 2 0 0 0-2 2v15a3 3 0 0 0 3 3h12a1 1 0 1 0 0-2h-2v-2h2a1 1 0 0 0 1-1V4a2 2 0 0 0-2-2h-8v16h5v2H7a1 1 0 1 1 0-2h1V2H6Z" clip-rule="evenodd"/>
          </svg>
        </div>
      <% end %>
    </div>

    <div>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Description</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@book.description}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Publisher</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@book.publisher.name}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Published</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@book.published}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Format</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@book.format}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Page Count</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@book.page_count}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Language</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@book.language}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Identifiers</dt>
        <%= for bid <- @book.bids do %>
          <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{bid.type}: {bid.bid}</dd>
        <% end %>
      </dl>
      <%= if not is_nil(@book.dbi_server) do %>
        <dl>
          <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">From Server</dt>
          <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">
            <%= if @current_user != nil and Role.can_moderate?(@current_user.role) do %>
              <.link navigate={~p"/servers/#{@book.dbi_server.id}/"} class="hover:underline">
                {@book.dbi_server.name}
              </.link>
            <% else %>
              <.link navigate={@book.dbi_server.url} class="hover:underline">
                {@book.dbi_server.name}
              </.link>
            <% end %>
          </dd>
        </dl>
      <% end %>
    </div>

    <%= if not Enum.empty?(@alternative_editions) do %>
      <h1 class="text-lg font-semibold leading-8 text-zinc-800 dark:text-white">Editions</h1>
      <div class="flex flex-wrap flex-[3_1_auto]">
        <%= for book <- @alternative_editions do %>
          <.book_card
            book={book}
            current_user={@current_user}
          />
        <% end %>
      </div>
    <% end %>

    <.back navigate={~p"/books"}>Back to books</.back>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok, socket}
  end

  @impl true
  def handle_params(%{"id" => id}, _, socket) do
    book =
      Ash.get!(Metadata.Book, id,
        actor: socket.assigns.current_user,
        load: [:bids, :author_roles, :publisher, :dbi_server]
      )

    alternative_editions = Metadata.get_book_alternative_editions!(book)

    {:noreply,
     socket
     |> assign(:page_title, page_title(socket.assigns.live_action))
     |> assign(:book, book)
     |> assign(:alternative_editions, alternative_editions)}
  end

  defp page_title(:show), do: "Show Book"
  defp page_title(:edit), do: "Edit Book"

  attr :book, :map, required: true

  def authors(assigns) do
    author_roles = Metadata.get_author_roles_by_book_id!(assigns.book.id, load: [:author])
    author_count = length(author_roles)

    authors_string =
      author_roles
      |> Enum.map(fn author_role ->
        if author_role.role in ["", nil] do
          {"#{author_role.author.name}", author_role.author.id}
        else
          {"#{author_role.author.name} (#{author_role.role})", author_role.author.id}
        end
      end)
      |> Enum.zip(1..author_count)

    assigns =
      assigns
      |> assign(:authors_string, authors_string)
      |> assign(:author_count, author_count)

    ~H"""
    <%= for {{name, id}, index} <- @authors_string do %>
      <.link navigate={~p"/authors/#{id}/"} class="hover:underline">
        {name}</.link><%= if {index} != {@author_count} do %>,<% end %>
    <% end %>
    """
  end
end
