defmodule DecentralisedBookIndexWeb.DbiServerLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata
  alias DecentralisedBookIndex.SyncTasks.SyncServerTask

  alias DecentralisedBookIndex.Accounts.Role

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      Listing Servers
      <:actions>
        <%= if @current_user != nil and Role.can_administrate?(@current_user.role) do %>
          <div class="flex flex-row gap-2">
            <div class="flex flex-row gap-2">
              <.primary_button phx-click="sync">
                Sync now
              </.primary_button>
            </div>
            <div class="flex flex-row gap-2">
              <.link patch={~p"/oban"}>
                <.primary_button>
                  Task dashboard
                </.primary_button>
              </.link>
            </div>
          </div>
        <% end %>
      </:actions>
    </.header>

    <div class="pt-2">
      <.search_resources
        search_query={@search_query}
        select_options={@select_options}
        selected_option={@sort_by}
        current_user={@current_user}
        resource_type="server"
        resource_new_url={~p"/servers/new"}
        moderator_role={:admin}
      />
    </div>

    <%= if Enum.empty?(@page.results) do %>
      <div class="flex justify-center ">
        <div>
          <p class="text-xl font-semibold py-5 dark:text-white">
            No Servers
          </p>
        </div>
      </div>
    <% else %>
      <div class="pt-2 flex flex-col gap-2">
        <.table
          id="dbi_servers"
          rows={@page.results}
          row_click={fn dbi_server -> JS.navigate(~p"/servers/#{dbi_server}") end}
        >
          <:col :let={dbi_server} label="Name">{dbi_server.name}</:col>

          <:col :let={dbi_server} label="Url">{dbi_server.url}</:col>

          <:action :let={dbi_server}>
            <div class="sr-only">
              <.link navigate={~p"/servers/#{dbi_server}"}>Show</.link>
            </div>

            <%= if @current_user != nil and Role.can_administrate?(@current_user.role) do %>
              <.link patch={~p"/servers/#{dbi_server}/edit"}>Edit</.link>
            <% end %>
          </:action>
        </.table>

        <.pagination
          endpoint={~p"/servers"}
          page={@page}
          page_params={@page_params}
          params={@params}
        />
      </div>
    <% end %>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    if connected?(socket) do
      Phoenix.PubSub.subscribe(DecentralisedBookIndex.PubSub, DecentralisedBookIndex.SyncWorkerManual.topic())
    end

    {:ok,
     socket
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "name") |> validate_sort_by()
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page =
      Metadata.search_dbi_server!(
        search_query,
        query: [sort_input: sort_by],
        page: page_params ++ [count: true],
        actor: socket.assigns.current_user
      )

    socket =
      socket
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:select_options, sort_options())
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:params, params)
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit server")
    |> assign(
      :dbi_server,
      Ash.get!(DecentralisedBookIndex.Metadata.DBIServer, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New server")
    |> assign(:dbi_server, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing servers")
    |> assign(:dbi_server, nil)
  end

  @impl true
  def handle_info(
        {DecentralisedBookIndexWeb.DbiServerLive.FormComponent, {:saved, _dbi_server}},
        socket
      ) do
    {:noreply, socket}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    dbi_server =
      Ash.get!(DecentralisedBookIndex.Metadata.DBIServer, id, actor: socket.assigns.current_user)

    Ash.destroy!(dbi_server, actor: socket.assigns.current_user)

    {:noreply, socket}
  end

  defp sort_options do
    [
      {"Sort by name", "name"},
      {"Sort by recently updated", "-updated_at"},
      {"Sort by recently added", "-inserted_at"}
    ]
  end

  def validate_sort_by(key) do
    valid_keys = Enum.map(sort_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end

  @impl true
  def handle_event("change-sort", %{"sort_by" => sort_by}, socket) do
    params =
      socket.assigns.params
      |> Map.put("sort_by", sort_by)

    {:noreply, push_patch(socket, to: ~p"/servers?#{params}")}
  end

  @impl true
  def handle_event("search", %{"query" => query}, socket) do
    params =
      socket.assigns.params
      |> Map.put("query", query)

    {:noreply, push_patch(socket, to: ~p"/servers?#{params}")}
  end

  @impl true
  def handle_event("sync", _params, socket) do
    case DecentralisedBookIndex.SyncWorkerManual.new(%{}) |> Oban.insert() do
      {:ok, _job} ->
        {:noreply, put_flash(socket, :info, "Task started")}

      {:error, reason} ->
        {:noreply, put_flash(socket, :error, "Failed to start task: #{inspect(reason)}")}
    end
  end

  @impl true
  def handle_info({:manual_sync_completed, result}, socket) do
    socket =
      if result == :ok do
        socket
        |> put_flash(:info, "Task completed")
      else
        socket
        |> put_flash(:error, "Task failed")
      end

    {:noreply, socket}
  end

  @impl true
  def handle_info(_params, socket) do
    {:noreply, socket}
  end
end
