defmodule DecentralisedBookIndex.Metadata.Forms.BookFormTest do
  use DecentralisedBookIndex.DataCase, async: true

  import ExUnitProperties

  alias DecentralisedBookIndex.Metadata

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "form submiting" do
    test "user can't submit" do
      user = generate(user())

      assert_raise Ash.Error.Forbidden, fn ->
        _form =
          AshPhoenix.Form.for_create(Metadata.Book, :create,
            as: "book",
            actor: user
          )
          |> AshPhoenix.Form.ensure_can_submit!()
      end
    end

    test "moderator can submit", %{user: user} do
      assert _form =
               AshPhoenix.Form.for_create(Metadata.Book, :create,
                 as: "book",
                 actor: user
               )
               |> AshPhoenix.Form.ensure_can_submit!()
    end
  end

  describe "create new book via the form" do
    test "with all required attributes", %{user: user} do
      bids = bids(actor: user)
      publisher = generate(publisher())
      author_roles = author_roles(actor: user)

      valid_params = %{
        "title" => "Book",
        "description" => "A cool author",
        "format" => "Paper",
        "language" => "English",
        "page_count" => 600,
        "published" => ~D[2025-03-06],
        "author_roles" => author_roles,
        "bids" => bids,
        "publisher_id" => publisher.id
      }

      form =
        AshPhoenix.Form.for_create(Metadata.Book, :create,
          as: "book",
          actor: user
        )

      assert {:ok, _book} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end

  describe "update existing book via the form" do
    test "update title, desciption and roles", %{user: user} do
      book = generate(book())

      valid_params = %{
        "title" => "Another Book",
        "description" => "A cool book",
        "author_roles" => author_roles()
      }

      form =
        AshPhoenix.Form.for_update(book, :update,
          as: "book",
          actor: user
        )

      assert {:ok, _book} = AshPhoenix.Form.submit(form, params: valid_params)
    end

    test "update bids and authors", %{user: user} do
      book = generate(book())
      bids = bids(actor: user)
      author_roles = author_roles(actor: user)

      valid_params = %{
        "author_roles" => author_roles,
        "bids" => bids
      }

      form =
        AshPhoenix.Form.for_update(book, :update,
          as: "book",
          actor: user
        )

      assert {:ok, _book} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end

  describe "validation" do
    property "the book form is valid if the page count > 0", %{user: user} do
      book = generate(book())
      author_roles = author_roles()

      valid_count_generator =
        StreamData.integer()
        |> StreamData.map(fn n -> abs(n) + 1 end)

      check all(page_count <- valid_count_generator) do
        valid_params = %{
          "page_count" => page_count,
          "author_roles" => author_roles
        }

        form =
          AshPhoenix.Form.for_update(book, :update,
            as: "book",
            actor: user
          )

        assert {:ok, _book} = AshPhoenix.Form.submit(form, params: valid_params)
      end
    end

    property "the book form is invalid if the page count <= 0", %{user: user} do
      book = generate(book())

      invalid_count_generator =
        StreamData.integer()
        |> StreamData.map(fn n -> -abs(n) end)

      check all(page_count <- invalid_count_generator) do
        invalid_params = %{
          "page_count" => page_count
        }

        form =
          AshPhoenix.Form.for_update(book, :update,
            as: "book",
            actor: user
          )

        assert {:error, _error} = AshPhoenix.Form.submit(form, params: invalid_params)
      end
    end
  end
end
