defmodule DecentralisedBookIndex.Sync.DataTransformers.BookSyncTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Sync.BookSync
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "sync book transformations" do
    test "a new book will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      publisher = generate(publisher())

      book = %{
        id: "1bbe8861-9d9d-4684-bda6-b6ec238d8d08",
        cover_image_url: "/images/book_cover.png",
        description: "A cool book.",
        format: "Paper",
        inserted_at: "2025-03-20T14:44:36.162986Z",
        language: "English",
        page_count: 1000,
        published: "2025-03-05",
        publisher_id: publisher.id,
        title: "Book",
        updated_at: "2025-04-01T18:14:25.754055Z"
      }

      assert :ok = BookSync.create_update(book, server.id)
      assert {:ok, saved_book} = Metadata.get_book_by_id(book.id)

      book =
        book
	|> datetime_from_iso8601_for_map()
        |> Map.replace(:published, Date.from_iso8601!(book[:published]))

      assert nil != saved_book.book_editions_registry_id
      assert server.id == saved_book.dbi_server_id
      assert get_submap(saved_book, book) == book
    end

    test "a new book has the same editions registry" do
      server = generate(dbi_server(url: @test_server_endpoint))

      publisher = generate(publisher())

      book = %{
        id: "1bbe8861-9d9d-4684-bda6-b6ec238d8d08",
        cover_image_url: "/images/book_cover.png",
        description: "A cool book.",
        format: "Paper",
        inserted_at: "2025-03-20T14:44:36.162986Z",
        language: "English",
        page_count: 1000,
        published: "2025-03-05",
        publisher_id: publisher.id,
        title: "Book",
        updated_at: "2025-04-01T18:14:25.754055Z",
        book_editions_registry_id: "1ed81d9e-3757-472a-aa3b-44fcdfbe770a"
      }

      assert :ok = BookSync.create_update(book, server.id)
      assert {:ok, saved_book} = Metadata.get_book_by_id(book.id)

      book =
        book
	|> datetime_from_iso8601_for_map()
        |> Map.replace(:published, Date.from_iso8601!(book[:published]))

      assert nil != saved_book.book_editions_registry_id
      assert server.id == saved_book.dbi_server_id
      assert get_submap(saved_book, book) == book
    end

    test "a new book with relationships will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      author1 = generate(author())
      author2 = generate(author())
      publisher = generate(publisher())

      book = %{
        author_roles: [
          %{
            id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8934",
            order: 0,
            role: nil,
            author_id: author1.id
          },
          %{
            id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8935",
            order: 1,
            role: "Translator",
            author_id: author2.id
          }
        ],
        bids: [
          %{
            bid: "1234567890123",
            id: "c6531191-ebc6-4a7c-af65-e2b5030a5147",
            order: 0,
            type: "isbn13"
          },
          %{
            bid: "1234567890",
            id: "c0238e74-845b-4d1f-94cf-fe16c0fa46a8",
            order: 1,
            type: "isbn10"
          }
        ],
        cover_image_url: nil,
        description: "Description 0",
        format: "Format 0",
        id: "fd09e57a-1f1e-4a80-b00a-398b3ee545fa",
        inserted_at: "2025-04-08T07:02:42.778095Z",
        language: "Language 0",
        page_count: 1,
        published: "2025-03-04",
        publisher_id: publisher.id,
        title: "Book 6",
        updated_at: "2025-04-08T07:02:42.778095Z"
      }

      assert :ok = BookSync.create_update(book, server.id)
      assert {:ok, saved_book} = Metadata.get_book_by_id(book.id, load: [:author_roles, :bids])

      bids = book[:bids]
      author_roles = book[:author_roles]

      assert get_ids(bids) == get_ids(saved_book.bids)
      assert get_ids(author_roles) == get_ids(saved_book.author_roles)

      assert get_submaps(saved_book.bids, List.first(bids)) == bids
      assert get_submaps(saved_book.author_roles, List.first(author_roles)) == author_roles
      assert book[:publisher_id] == saved_book.publisher_id
    end

    test "update an existing book" do
      server = generate(dbi_server(url: @test_server_endpoint))

      book = generate(book())

      book_attrs = %{
        id: book.id,
        cover_image_url: "/images/book_cover2.png",
        description: "A cool book 2.",
        format: "Ebook",
        inserted_at: "2025-01-20T14:44:36.162986Z",
        language: "English",
        page_count: 1001,
        published: "2025-03-05",
        title: "Book2",
        updated_at: "2025-02-01T18:14:25.754055Z"
      }

      assert :ok = BookSync.create_update(book_attrs, server.id)
      assert {:ok, _saved_book} = Metadata.get_book_by_id(book.id)
    end

    test "the local existing book wouldn't updated" do
      server = generate(dbi_server(url: @test_server_endpoint))

      book = generate(book())

      book_attrs = %{
        id: book.id,
        cover_image_url: "/images/book_cover2.png",
        description: "A cool book 2.",
        format: "Ebook",
        inserted_at: "2025-01-20T14:44:36.162986Z",
        language: "English",
        page_count: 1001,
        published: "2025-03-05",
        title: "New Book2",
        updated_at: "2025-02-01T18:14:25.754055Z"
      }

      assert :ok = BookSync.create_update(book_attrs, server.id)
      assert {:ok, saved_book} = Metadata.get_book_by_id(book.id)

      assert get_submap(saved_book, book_attrs) != book_attrs
      assert saved_book.title != "New Book2"
      assert saved_book.dbi_server_id == nil
    end

    test "old book wouldn't be sync" do
      server = generate(dbi_server(url: @test_server_endpoint))

      book = generate(book())

      book_attrs = %{
        id: book.id,
        cover_image_url: "/images/book_cover2.png",
        description: "A cool book 2.",
        format: "Ebook",
        inserted_at: "2025-01-20T14:44:36.162986Z",
        language: "English",
        page_count: 1001,
        published: "2025-03-05",
        title: "New Book2",
        updated_at: "2025-02-01T18:14:25.754055Z"
      }

      assert :ok = BookSync.create_update(book_attrs, server.id)
      assert {:ok, saved_book} = Metadata.get_book_by_id(book.id)

      assert get_submap(saved_book, book_attrs) != book_attrs
      assert saved_book.title != "New Book2"
    end

    test "update book's relationship for an existing book" do
      server = generate(dbi_server(url: @test_server_endpoint))

      book = generate(book(dbi_server_id: server.id))

      author1 = generate(author())
      author2 = generate(author())
      publisher = generate(publisher())

      book_attrs = %{
        author_roles: [
          %{
            id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8934",
            order: 0,
            role: "",
            author_id: author1.id
          },
          %{
            id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8935",
            order: 1,
            role: "Translator",
            author_id: author2.id
          }
        ],
        bids: [
          %{
            bid: "1234567890123",
            id: "c6531191-ebc6-4a7c-af65-e2b5030a5147",
            order: 0,
            type: "isbn13"
          },
          %{
            bid: "1234567890",
            id: "c0238e74-845b-4d1f-94cf-fe16c0fa46a8",
            order: 1,
            type: "isbn10"
          }
        ],
        cover_image_url: nil,
        description: "Description 0",
        format: "Format 0",
        id: book.id,
        inserted_at: "2025-04-08T07:02:42.778095Z",
        language: "Language 0",
        page_count: 1,
        published: "2025-03-04",
        publisher_id: publisher.id,
        title: "Book 6",
        updated_at: datetime_add_second_as_string(book.updated_at, 10),
        dbi_server_id: server.id
      }

      assert :ok = BookSync.create_update(book_attrs, server.id)
      assert {:ok, saved_book} = Metadata.get_book_by_id(book.id, load: [:author_roles, :bids])

      bids = book_attrs[:bids]
      author_roles = book_attrs[:author_roles]

      assert get_ids(bids) == get_ids(saved_book.bids)
      assert get_ids(author_roles) == get_ids(saved_book.author_roles)
    end

    test "olds bids and roles are destroyed after update" do
      server = generate(dbi_server(url: @test_server_endpoint))

      book = generate(book(dbi_server_id: server.id))

      author1 = generate(author())
      author2 = generate(author())
      publisher = generate(publisher())

      book_attrs = %{
        author_roles: [
          %{
            id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8934",
            order: 0,
            role: "",
            author_id: author1.id
          },
          %{
            id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8935",
            order: 1,
            role: "Translator",
            author_id: author2.id
          }
        ],
        bids: [
          %{
            bid: "1234567890123",
            id: "c6531191-ebc6-4a7c-af65-e2b5030a5147",
            order: 0,
            type: "isbn13"
          },
          %{
            bid: "1234567890",
            id: "c0238e74-845b-4d1f-94cf-fe16c0fa46a8",
            order: 1,
            type: "isbn10"
          }
        ],
        cover_image_url: nil,
        description: "Description 0",
        format: "Format 0",
        id: book.id,
        inserted_at: "2025-04-08T07:02:42.778095Z",
        language: "Language 0",
        page_count: 1,
        published: "2025-03-04",
        publisher_id: publisher.id,
        title: "Book 6",
        updated_at: datetime_add_second_as_string(book.updated_at, 10),
        dbi_server_id: server.id
      }

      old_bids = get_ids(Metadata.list_book_ids!())
      old_roles = get_ids(Metadata.list_author_roles!())

      assert :ok = BookSync.create_update(book_attrs, server.id)
      assert {:ok, _saved_book} = Metadata.get_book_by_id(book.id, load: [:author_roles, :bids])

      new_bids = get_ids(Metadata.list_book_ids!())
      new_roles = get_ids(Metadata.list_author_roles!())

      assert Enum.all?(old_bids, fn id -> id not in new_bids end)
      assert Enum.all?(old_roles, fn id -> id not in new_roles end)
    end
  end
end
